/////////////////////////////////////////////////////////////////////////////////
// File:     leakoverflow.c 
// Purpose:  example code that has a memory leak and a
//           buffer overflow error that we can catch
//           with Valgrind
//
// Compile:  gcc -Wall -g -o leakoverflow leakoverflow.c
//
// Author:   Eric Kee, copied from 
//           http://valgrind.org/docs/manual/quick-start.html
//
// Class:    CS78, Computer Networks, Spring 2008
//
// Comments:
//   This program demonstrates how valgrind can be used
//   to catch memory leaks and buffer overflows.  To
//   test, compile this program as specified above and 
//   then run the program with Valgrind, as follows:
//
//     valgrind --leak-check=yes ./leakoverflow
//
//   You should see output that looks something like
//   this:
//
//   [tahoe:~] 50) valgrind --leak-check=yes ./leakoverflow
//   ==30120== Memcheck, a memory error detector.
//   ==30120== Copyright (C) 2002-2007, and GNU GPL'd, by Julian Seward et al.
//   ==30120== Using LibVEX rev 1732, a library for dynamic binary translation.
//   ==30120== Copyright (C) 2004-2007, and GNU GPL'd, by OpenWorks LLP.
//   ==30120== Using valgrind-3.2.3, a dynamic binary instrumentation framework.
//   ==30120== Copyright (C) 2000-2007, and GNU GPL'd, by Julian Seward et al.
//   ==30120== For more details, rerun with: -v
//   ==30120== 
//   ==30120== Invalid write of size 4
//   ==30120==    at 0x80483DF: f (leakoverflow.c:70)
//   ==30120==    by 0x80483FC: main (leakoverflow.c:76)
//   ==30120==  Address 0x41A3050 is 0 bytes after a block of size 40 alloc'd
//   ==30120==    at 0x4022525: malloc (vg_replace_malloc.c:149)
//   ==30120==    by 0x80483D5: f (leakoverflow.c:68)
//   ==30120==    by 0x80483FC: main (leakoverflow.c:76)
//   ==30120== 
//   ==30120== ERROR SUMMARY: 1 errors from 1 contexts (suppressed: 13 from 1)
//   ==30120== malloc/free: in use at exit: 40 bytes in 1 blocks.
//   ==30120== malloc/free: 1 allocs, 0 frees, 40 bytes allocated.
//   ==30120== For counts of detected errors, rerun with: -v
//   ==30120== searching for pointers to 1 not-freed blocks.
//   ==30120== checked 47,932 bytes.
//   ==30120== 
//   ==30120== 
//   ==30120== 40 bytes in 1 blocks are definitely lost in loss record 1 of 1
//   ==30120==    at 0x4022525: malloc (vg_replace_malloc.c:149)
//   ==30120==    by 0x80483D5: f (leakoverflow.c:68)
//   ==30120==    by 0x80483FC: main (leakoverflow.c:76)
//   ==30120== 
//   ==30120== LEAK SUMMARY:
//   ==30120==    definitely lost: 40 bytes in 1 blocks.
//   ==30120==      possibly lost: 0 bytes in 0 blocks.
//   ==30120==    still reachable: 0 bytes in 0 blocks.
//   ==30120==         suppressed: 0 bytes in 0 blocks.
//
////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h>

void f(void)
{
  int* x;

  x = malloc(10 * sizeof(int));
                     // problem 1: heap block overrun
  x[10] = 0;         // problem 2: memory leak -- x not freed
}                    


int main(void)
{
  f();
  printf("Everything is working perfectly!\n");
  return 0;
}
